#!/bin/bash
# Copyright 2016 SpeechLab at FEE CTU Prague (Author: Jiri Fiala, Petr Mizera)
# Apache 2.0

tool="compute-mfcc"
#   - compute-mfcc or ctucopy
. ./utils/parse_options.sh

if [ $# -le 1 ]; then
   echo "Arguments should be a list of database directories, see ../run.sh for example."
   exit 1;
fi

database=$1
language=$2
dir_work=`pwd`

dir=$dir_work/data/$language/local

mkdir -p $dir

database=$database/$language/FIXED3$language
content=$database/INDEX/CONTENTS.LST

. ./path.sh # Needed for KALDI_ROOT

##################
# Filtering utterances with word truncations, mispronunciations, non-understandable speech
# get speaker gender
tail -n +2 $content | perl -nle '@a=split;$line=join "\t",@a;$_=join " ",@a[8..@a]; if(!/\.|~|\*|\%|\-|\/|\_/){ print $line}' > $dir/filtered_content

if [ $language == "HU"  ]; then
  cut -d$'\t' -f1,7 $database/TABLE/SESSION.TBL | tail -n +2 > $dir/all.gender
else
  cut -d$'\t' -f1,6 $database/TABLE/SESSION.TBL | tail -n +2 > $dir/all.gender
fi

#get test session to test and dev
test_lines_count=`wc -l $database/INDEX/A3TST${language}.SES | sed 's/\s.*$//'`
lines_to_split=`echo "$test_lines_count / 2" | bc`
head -n $lines_to_split $database/INDEX/A3TST${language}.SES | tr -d '\r' > $dir/spk_test
tail -n $lines_to_split $database/INDEX/A3TST${language}.SES | tr -d '\r' > $dir/spk_dev

if [ -f ${database}/INDEX/A3TRN${language}.SES ]; then
  cat  ${database}/INDEX/A3TRN${language}.SES | tr -d '\r' > $dir/spk_train
else
  cut -d$'\t' -f1 $database/TABLE/SESSION.TBL | tail -n +2 | tr -d '\r' > $dir/${language}.all_ses
  grep -v -f $dir/spk_test $dir/${language}.all_ses | tr -d '\r'> $dir/spk_train
fi

for data_set in train test dev; do
  echo $data_set
  echo "Split the speaker to train, test and dev"
  grep -f $dir/spk_${data_set} $dir/all.gender | sort -u | tr '[:upper:]' '[:lower:]'  > $dir/${data_set}.gender

  cut -d$'\t' -f3  $dir/filtered_content | sed s/.${language}A//g | grep -f $dir/spk_${data_set} \
      | awk '{print $1," " substr ($1, 3, 4)}' | sort -u > $dir/${data_set}.utt2spk
  perl utils/utt2spk_to_spk2utt.pl $dir/${data_set}.utt2spk | sort -u > $dir/${data_set}.spk2utt

  if [ $tool ==  "ctucopy" ]; then
     cut -d$'\t' -f2,3 $dir/filtered_content | tr '\t' '/' | sed 's/\\/\//g' | sed 's/^\/FIXED...\///g' |grep -f $dir/spk_${data_set} \
        | awk -v var=$database '{print " "var"/"$1""}' | sort -u > $dir/${data_set}_part.scp
  else
     cut -d$'\t' -f2,3 $dir/filtered_content | tr '\t' '/' | sed 's/\\/\//g' | sed 's/^\/FIXED...\///g' |grep -f $dir/spk_${data_set} \
        | awk -v var=$database '{print "sox -t raw -r 8000 -e a-law -c 1 "var"/"$1" -t wav -r 8000 -e signed-integer -c 1 - |"}' \
        | sort -u > $dir/${data_set}_part.scp
  fi
  paste $dir/${data_set}.utt2spk $dir/${data_set}_part.scp | awk '{$2=""; print $0}' > $dir/${data_set}.scp

  echo "Create transcripts files"
  perl local/speechdat_create_trans.pl $dir/${data_set}.utt2spk  $database $language | sort -k1  > $dir/${data_set}.txt

  [ -d $dir_work/data/$language/${data_set} ] && rm -r $dir_work/data/$language/${data_set}
  mkdir -p $dir_work/data/$language/${data_set}
done

# Final data/train,dev,test directories
data_set="train"
cp $dir/${data_set}.scp $dir_work/data/$language/${data_set}/wav.scp || exit 1;
cp $dir/${data_set}.txt $dir_work/data/$language/${data_set}/text || exit 1;
cp $dir/${data_set}.spk2utt $dir_work/data/$language/${data_set}/spk2utt || exit 1;
cp $dir/${data_set}.utt2spk $dir_work/data/$language/${data_set}/utt2spk || exit 1;
cp $dir/${data_set}.gender $dir_work/data/$language/${data_set}/spk2gender || exit 1;

# text.lm for LM
cat $dir_work/data/$language/${data_set}/text | grep 'S[0-9]\|Z[0-9]' | cut -d' ' -f2- \
  | sed  -e "s/ / /g" -e "s/ $//g" | sort | uniq | sed -e "s/^/<s> /" -e "s/$/ <\/s>/" > $dir_work/data/$language/${data_set}/text.lm

# only sentences for recognition task
for data_set in test dev; do
   cat $dir/${data_set}.scp | grep "Z[0-9].${language}A\|S[0-9].${language}A" > $dir_work/data/$language/${data_set}/wav.scp
   cat $dir/${data_set}.txt | grep 'S[0-9]\|Z[0-9]' > $dir_work/data/$language/${data_set}/text
   cat $dir/${data_set}.utt2spk | grep 'S[0-9]\|Z[0-9]' > $dir_work/data/$language/${data_set}/utt2spk
   perl utils/utt2spk_to_spk2utt.pl $dir_work/data/$language/${data_set}/utt2spk > $dir_work/data/$language/${data_set}/spk2utt
   cut -d' ' -f1 $dir_work/data/$language/${data_set}/spk2utt > $dir_work/data/$language/${data_set}/tmp_spk
   grep -f $dir_work/data/$language/${data_set}/tmp_spk $dir/${data_set}.gender  > $dir_work/data/$language/${data_set}/spk2gender
   [ -f $dir_work/data/$language/${data_set}/tmp_spk ] && rm $dir_work/data/$language/${data_set}/tmp_spk
done

echo "Data preparation finished"
