#!/bin/bash

# Copyright 2016 SpeechLab at FEE CTU Prague (Author: Petr Mizera, Jiri Fiala)
# Apache 2.0
# Note: This file is based on wsj/s5/local/wsj_prepare_dict.sh

# The parts of the output of this that will be needed are
# [in data/local/dict/ ]
# lexicon.txt
# extra_questions.txt
# nonsilence_phones.txt
# optional_silence.txt
# silence_phones.txt

# run this from ../

xsampa_map=true
extra_questions=true

. ./utils/parse_options.sh

database=$1;shift
language=$1;shift
dir_work=`pwd`
dir=$dir_work/data/$language/local/dict
database=$database/$language/FIXED3$language

[ -f path.sh ] && . ./path.sh
[ -d $dir ] && rm -r $dir
mkdir -p $dir

# Dictionary preparation:
  echo sil > $dir/silence_phones.txt
  echo sil > $dir/optional_silence.txt

  # SPEECHDAT dictionary
  if [ $language == "HU"  ]; then
   cut -d$'\t' -f1,2- $database/TABLE/LEXICON.TBL | grep -v Orthography | tail -n +2 | tr -d $'\r' > $dir/lexicon2.txt
  else
   cut -d$'\t' -f1,3- $database/TABLE/LEXICON.TBL | grep -v Orthography | tail -n +2 | tr -d $'\r#' > $dir/lexicon2.txt
  fi

  if $xsampa_map; then
    # Use the following to map to X-SAMPA phoneset
    perl local/speechdat_prepare_dict_${language}.pl $dir/lexicon2.txt > $dir/lexicon.txt
  else
    perl local/speechdat_prepare_dict.pl $dir/lexicon2.txt > $dir/lexicon.txt
  fi
  cut -f 2- -d " " $dir/lexicon.txt | sed "s/ /\n/g" | sed '/^$/d' | sort -u > $dir/phones.txt

  echo "!sil sil" >> $dir/lexicon.txt
  sort -u -o $dir/lexicon.txt $dir/lexicon.txt
  grep -v -F -f $dir/silence_phones.txt $dir/phones.txt > $dir/nonsilence_phones.txt
  echo $dir/lexicon.txt

  if $extra_questions; then
    # Ask questions about the entire set of 'silence' and 'non-silence' phones.
    # These augment the questions obtained automatically by clustering.
    ( tr '\n' ' ' < $dir/silence_phones.txt; echo;
      tr '\n' ' ' < $dir/nonsilence_phones.txt; echo;
    ) > $dir/extra_questions.txt
  else
    echo "file with extra_questions.txt is empty"
    touch $dir/extra_questions.txt
  fi

echo "Dictionary preparation succeeded"
