#!/bin/bash

# Copyright 2016 SpeechLab at FEE CTU Prague (Author: Jiri Fiala, Petr Mizera)
# Apache 2.0
# The example script for the SPEECHDAT .

. ./cmd.sh ## You'll want to change cmd.sh to something that will work on your system.
           ## This relates to the queue.

[ -f path.sh ] && . ./path.sh
set -e

# Set the path to the root speecon database directory
speechdat=/data/SPEECHDAT

# Choose language
lang="CS"
#       - CS HU RU PL SK

# stage of run.sh
stage=0;
#      0 - Data & Lexicon & Language Preparation
#      1 - Feature Extration
#      2 - AM training
#      3 - Create HCLG.fst
#      4 - Decode

tool="ctucopy"
#    - compute-mfcc # KALDI mfcc feature extraction tool
#    - ctucopy      # CtuCopy tool implementing speech enhancement and feature extraction algorithms.

# Use the following to map to X-SAMPA phoneset
xsampa_map=true

if [ $stage == 0 ]; then
 echo "Data & Lexicon & Language Preparation"
 local/speechdat_data_prep.sh --tool $tool $speechdat $lang
 local/speechdat_prep_dict.sh --xsampa_map $xsampa_map $speechdat $lang
 utils/prepare_lang.sh data/$lang/local/dict "!sil" data/$lang/local/lang_tmp data/$lang/lang || exit 1;
 local/create_LM.sh  --smoothing "-wbdiscount" data/$lang/train/text.lm data/$lang/local/lm || exit 1;
 local/speechdat_format_lms.sh --lm-srcdir "data/$lang/local/lm" data/$lang/lang
fi

if [ $stage == 1 ]; then
 echo "Feature Extration & CMN stats for Training and Test set"
 mfccdir=/scratch/fiala/speechdat/mfcc/$lang
 for x in train dev test; do
   local/make_fea_ctucopy.sh --cmd "$train_cmd" --nj 32 data/$lang/$x exp/mfcc/$lang/$x $mfccdir/$x || exit 1;
#   steps/make_mfcc.sh --cmd "$train_cmd" --nj 32  data/$lang/$x exp/mfcc/$lang/$x $mfccdir || exit 1;
   steps/compute_cmvn_stats.sh data/$lang/$x exp/mfcc/$lang/$x $mfccdir/$x || exit 1;
 done
fi

if [ $stage == 2 ]; then
 echo "MonoPhone Training & Decoding"
 steps/train_mono.sh  --nj "$train_nj" --cmd "$train_cmd" data/$lang/train data/$lang/lang exp/$lang/mono || exit 1;
exit
 echo "tri1 : Deltas + Delta-Deltas Training & Decoding"
 steps/align_si.sh --boost-silence 1.25 --nj "$train_nj" --cmd "$train_cmd" data/$lang/train data/$lang/lang exp/$lang/mono exp/$lang/mono_ali || exit 1;
 steps/train_deltas.sh --cmd "$train_cmd" 2500 15000 data/$lang/train data/$lang/lang exp/$lang/mono_ali exp/$lang/tri1 || exit 1;

 echo "tri2 : LDA + MLLT Training & Decoding"
 steps/align_si.sh --nj "$train_nj" --cmd "$train_cmd" data/$lang/train data/$lang/lang exp/$lang/tri1 exp/$lang/tri1_ali || exit 1;
 steps/train_lda_mllt.sh --cmd "$train_cmd" --splice-opts "--left-context=3 --right-context=3" 2500 \
          15000 data/$lang/train data/$lang/lang exp/$lang/tri1_ali exp/$lang/tri2 || exit 1;

 echo "tri3 : LDA + MLLT + SAT Training & Decoding"
 steps/align_si.sh --nj "$train_nj" --cmd "$train_cmd" --use-graphs true data/$lang/train data/$lang/lang exp/$lang/tri2 exp/$lang/tri2_ali || exit 1;
 steps/train_sat.sh --cmd "$train_cmd" 4200 40000 data/$lang/train data/$lang/lang exp/$lang/tri2_ali exp/$lang/tri3 || exit 1;

 echo "SGMM2 Training & Decoding"
 steps/align_fmllr.sh --nj "$train_nj" --cmd "$train_cmd" data/$lang/train data/$lang/lang exp/$lang/tri3 exp/$lang/tri3_alia || exit 1;
 steps/train_ubm.sh --cmd "$train_cmd" 400 data/$lang/train data/$lang/lang exp/$lang/tri3_alia exp/$lang/ubm4a || exit 1;
 steps/train_sgmm2.sh --cmd "$train_cmd" 7000 9000 data/$lang/train data/$lang/lang exp/$lang/tri3_alia exp/$lang/ubm4a/final.ubm exp/$lang/sgmm2_4a || exit 1;

 echo "MMI + SGMM2 Training & Decoding"
  steps/align_sgmm2.sh --nj "$train_nj" --cmd "$train_cmd" --transform-dir exp/$lang/tri3_alia --use-graphs true --use-gselect true \
                        data/$lang/train data/$lang/lang exp/$lang/sgmm2_4a exp/$lang/sgmm2_4_alia || exit 1;

   steps/make_denlats_sgmm2.sh --nj "$train_nj" --sub-split "$train_nj" --acwt 0.2 --lattice-beam 10.0 --beam 18.0 --cmd "$train_cmd" \
                        --transform-dir exp/$lang/tri3_alia \
                        data/$lang/train data/$lang/lang exp/$lang/sgmm2_4_alia exp/$lang/sgmm2_4_denlatsa || exit 1;

   steps/train_mmi_sgmm2.sh --acwt 0.2 --cmd "$train_cmd" --transform-dir exp/$lang/tri3_alia --boost 0.1 --drop-frames true \
                        data/$lang/train data/$lang/lang exp/$lang/sgmm2_4_alia exp/$lang/sgmm2_4_denlatsa exp/$lang/sgmm2_4_mmi_b0.1a || exit 1;
fi

if [ $stage == 3 ]; then
   echo "Create HCLG.fst graphs"
   (
   $train_cmd JOB=1:1 exp/$lang/mono/log/mkgraph.lm_tg.log \
      utils/mkgraph.sh --mono data/$lang/lang_test_lm_tg exp/$lang/mono exp/$lang/mono/graph_lm_tg || exit 1;
   )&

   for x in tri1 tri2 tri3 sgmm2_4a; do
     (
      $train_cmd JOB=1:1 exp/$lang/$x/log/mkgraph.lm_tg.log \
         utils/mkgraph.sh data/$lang/lang_test_lm_tg exp/$lang/$x exp/$lang/$x/graph_lm_tg || exit 1;
     )&
   done
   wait
fi

if [ $stage == 4 ]; then
   echo "Decode"
   for eval in test dev; do
     (
      steps/decode.sh --nj "$decode_nj" --cmd "$decode_cmd" \
                      exp/$lang/mono/graph_lm_tg \
                      data/$lang/${eval} \
                      exp/$lang/mono/decode_${eval}_lm_tg || exit 1;

      steps/decode.sh --nj "$decode_nj" --cmd "$decode_cmd" \
                      exp/$lang/tri1/graph_lm_tg \
                      data/$lang/${eval} \
                      exp/$lang/tri1/decode_${eval}_lm_tg || exit 1;

      steps/decode.sh --nj "$decode_nj" --cmd "$decode_cmd" \
                      exp/$lang/tri2/graph_lm_tg \
                      data/$lang/${eval} \
                      exp/$lang/tri2/decode_${eval}_lm_tg || exit 1;

      steps/decode_fmllr.sh --nj "$decode_nj" --cmd "$decode_cmd" \
                      exp/$lang/tri3/graph_lm_tg \
                      data/$lang/${eval} \
                      exp/$lang/tri3/decode_${eval}_lm_tg || exit 1;

      steps/decode_sgmm2.sh --nj "$decode_nj" --cmd "$decode_cmd" \
                      --transform-dir exp/$lang/tri3/decode_${eval}_lm_tg \
                      exp/$lang/sgmm2_4a/graph_lm_tg \
                      data/$lang/${eval} \
                      exp/$lang/sgmm2_4a/decode_${eval}_lm_tg || exit 1;

      for iter in 1 2 3 4; do
        (
        steps/decode_sgmm2_rescore.sh --cmd "$decode_cmd" \
                      --iter $iter \
                      --transform-dir exp/$lang/tri3/decode_${eval}_lm_tg \
                      data/$lang/lang_test_lm_tg \
                      data/$lang/${eval} \
                      exp/$lang/sgmm2_4a/decode_${eval}_lm_tg \
                      exp/$lang/sgmm2_4_mmi_b0.1a/decode_${eval}_lm_tg$iter || exit 1;
        )&
      done
    )&
   done
fi
