#!/bin/bash
# Decode language with ML AM

. ./cmd.sh
. ./path.sh

stages="2 3"
# 0 - prepare data
# 1 - create HCLG
# 2 - decode
# 3 - print results

languages="CS SK HU RU PL"
train_nj=15

# filter language data from ml
for language in $languages; do
    for stage in $stages; do
        if [ $stage == 0 ]; then
            echo "data preparation"    
            [ -d data/${language}_ML/ ] && rm -r data/${language}_ML/
            for dataset in test dev; do
                mkdir -p data/${language}_ML/$dataset
                cat data/ml/$dataset/text | grep $language > data/${language}_ML/$dataset/text
                cat data/ml/$dataset/spk2utt | grep $language > data/${language}_ML/$dataset/spk2utt
                cat data/ml/$dataset/utt2spk | grep $language > data/${language}_ML/$dataset/utt2spk
                cat data/ml/$dataset/wav.scp | grep $language > data/${language}_ML/$dataset/wav.scp
                cat data/ml/$dataset/spk2gender | grep $language >  data/${language}_ML/$dataset/spk2gender
                cat data/ml/$dataset/feats.scp | grep $language > data/${language}_ML/$dataset/feats.scp
                cat data/ml/$dataset/cmvn.scp | grep $language > data/${language}_ML/$dataset/cmvn.scp
            done

            # Create test dict and lang directory
            [ -d data/${language}_ML/local/dict ] && rm -r data/${language}_ML/local/dict
            mkdir -p data/${language}_ML/local/dict

            cp data/$language/local/dict/lexicon.txt    data/${language}_ML/local/dict/
            cp data/ml/local/dict/nonsilence_phones.txt data/${language}_ML/local/dict/
            cp data/ml/local/dict/optional_silence.txt  data/${language}_ML/local/dict/
            cp data/ml/local/dict/silence_phones.txt    data/${language}_ML/local/dict/
            cp data/ml/local/dict/extra_questions.txt   data/${language}_ML/local/dict/

            [ -d data/${language}_ML/local/lang_tmp ] && rm -r data/${language}_ML/local/lang_tmp
            [ -d data/${language}_ML/lang  ] && rm -r data/${language}_ML/lang
            
            utils/prepare_lang.sh data/${language}_ML/local/dict/ "!SIL" \
            data/${language}_ML/local/lang_tmp data/${language}_ML/lang
            cp data/${language}/lang_test_lm_tg/G.fst data/${language}_ML/lang
        fi

        if [ $stage == 1 ]; then
            echo "Create HCLG.fst graphs"
            (
            # using lang dir for the particular language: data/$language/lang_test_lm_tg
            # using the model from the multilingual system:  exp_ml/ml/mono
            # creating the graph for the particular language: exp/ml_$language/mono/graph_lm_tg

            $big_memory_cmd JOB=1:1 exp/ml_$language/mono/log/mkgraph_${language}_ML.lm_tg.log \
                    utils/mkgraph.sh --mono data/${language}_ML/lang \
                                     exp/ml/mono \
                                     exp/ml/mono/graph_lm_${language}_ML || exit 1;
            )&

            for x in tri1 tri2 tri3 sgmm2_4a; do
                (
                $big_memory_cmd JOB=1:1 exp/ml_$language/$x/log/mkgraph.lm_tg.log \
                    utils/mkgraph.sh data/${language}_ML/lang \
                                     exp/ml/$x\
                                     exp/ml/$x/graph_lm_${language}_ML || exit 1;
                )&
            done
            wait
            echo "Graph created successfully"
        fi

        if [ $stage == 2 ]; then
            echo "Decode"
            for eval in test dev; do
                steps/decode.sh  --nj "$decode_nj" --cmd "$decode_cmd" \
                                  exp/ml/mono/graph_lm_${language}_ML \
                                  data/${language}_ML/$eval \
                                  exp/ml/mono/decode_${eval}_${language}_ML

                steps/decode.sh --nj "$decode_nj" --cmd "$decode_cmd" \
                                  exp/ml/tri1/graph_lm_${language}_ML\
                                  data/${language}_ML/$eval \
                                  exp/ml/tri1/decode_${eval}_${language}_ML 

                steps/decode.sh  --nj "$decode_nj" --cmd "$decode_cmd" \
                                  exp/ml/tri2/graph_lm_${language}_ML\
                                  data/${language}_ML/$eval \
                                  exp/ml/tri2/decode_${eval}_${language}_ML || exit 1;

                steps/decode_fmllr.sh  --nj "$decode_nj" --cmd "$decode_cmd" \
                                  exp/ml/tri3/graph_lm_${language}_ML\
                                  data/${language}_ML/$eval \
                                  exp/ml/tri3/decode_${eval}_${language}_ML
                                
                steps/decode_sgmm2.sh --nj "$decode_nj" --cmd "$decode_cmd" \
                                  --transform-dir exp/ml/tri3/decode_${eval}_${language}_ML \
                                  exp/ml/sgmm2_4a/graph_lm_${language}_ML \
                                  data/${language}_ML/$eval \
                                  exp/ml/sgmm2_4a/decode_${eval}_${language}_ML || exit 1;

                    for iter in 1 2 3 4; do
                        (
                        steps/decode_sgmm2_rescore.sh --cmd "$decode_cmd" \
                                  --iter $iter \
                                  --transform-dir  exp/ml/tri3/decode_${eval}_${language}_ML\
                                  exp/ml/sgmm2_4a/graph_lm_${language}_ML \
                                  data/${language}_ML/$eval \
                                  exp/ml/sgmm2_4a/decode_${eval}_${language}_ML \
                                  exp/ml/sgmm2_4_mmi_b0.1a/decode_${eval}_${language}_ML$iter || exit 1;
                        )&
                    done
                  wait
             done
        fi

        # Print best WER
        if [ $stage == 3 ]; then
        for model in mono tri1 tri2 tri3 sgmm2_4a sgmm2_4_mmi_b0.1a; do
            for subset in test dev; do
                grep WER exp/ml/${model}/decode_${subset}_${language}_ML*/wer_* | utils/best_wer.sh
            done
        done
        fi
    done
done
