#!/bin/bash

set -o errexit

function error_exit () {
  echo -e "$@" >&2; exit 1;
}

if [ $# -le 0 ]; then
   echo "Missing arguments. see ../run.sh for example."
   exit 1;
fi

database=$1;shift
language=$1;shift
dir_work=$1;shift

dir=$dir_work/data/$language/local
dir_dict=$dir/dict

mkdir -p $dir
mkdir -p $dir_dict

local=`pwd`/local
utils=`pwd`/utils
ref=`pwd`/ref
conf=`pwd`/conf/speechdat

database=$database/$language/FIXED3$language
content=$database/INDEX/CONTENTS.LST
lang_lower=`echo $language | tr '[:upper:]' '[:lower:]'`
#dictionary=/storage/brno9-ceitec/home/fialaji8/speechdat_ml/dict/$language/spdat/fixed3${lang_lower}.xdict
dictionary=$database/TABLE/LEXICON.TBL
. ./path.sh # Needed for KALDI_ROOT

##################
# Filtering utterances with word truncations, mispronunciations, non-understandable speech
# get speaker gender
tail -n +2 $content | perl -nle '@a=split;$line=join "\t",@a;$_=join " ",@a[8..@a]; if(!/\.|~|\*|\%|\-|\/|\_/){ print $line}' > $dir/filtered_content

if [ $language == "HU"  ]; then
cut -d$'\t' -f1,7 $database/TABLE/SESSION.TBL | tail -n +2 > $dir/all.gender 
else
cut -d$'\t' -f1,6 $database/TABLE/SESSION.TBL | tail -n +2 > $dir/all.gender 
fi

perl local/speechdat_prepare_dict_${language}.pl $dictionary > $dir_dict/lexicon.txt
perl local/speechdat_prepare_dict_${language}.pl $dictionary 1.0 > $dir_dict/lexiconp.txt
#echo "!SIL sil" > $dir_dict/lexicon.txt
#cat $dictionary | tr -d '\r' | sed 's/\t/ /g' >> $dir_dict/lexicon.txt
#echo "!SIL 1.0  sil" > $dir_dict/lexiconp.txt
#cat $dictionary | tr -d '\r' |sed 's/\t/ 1.0 /g' >> $dir_dict/lexiconp.txt

cat $dir_dict/lexicon.txt | awk '{print $1="",$0}' | sed "s/ /\n/g" | sed '/^$/d' | sort -u > $dir_dict/phones.txt
cat $dir_dict/phones.txt | grep "sil" > $dir_dict/silence_phones.txt
cat $dir_dict/phones.txt | grep  -v "sil" > $dir_dict/nonsilence_phones.txt
echo "sil" > $dir_dict/optional_silence.txt
( tr '\n' ' ' < $dir_dict/silence_phones.txt; echo;
  tr '\n' ' ' < $dir_dict/nonsilence_phones.txt; echo;
) > $dir_dict/extra_questions.txt

#get test session to test and dev
test_lines_count=`wc -l $database/INDEX/A3TST${language}.SES | sed 's/\s.*$//'`
lines_to_split=`echo "$test_lines_count / 2" | bc`
head -n $lines_to_split $database/INDEX/A3TST${language}.SES | tr -d '\r' > $dir/spk_test
tail -n $lines_to_split $database/INDEX/A3TST${language}.SES | tr -d '\r' > $dir/spk_dev

if [ -f ${database}/INDEX/A3TRN${language}.SES ]; then
cat  ${database}/INDEX/A3TRN${language}.SES | tr -d '\r' > $dir/spk_train
else
cut -d$'\t' -f1 $database/TABLE/SESSION.TBL | tail -n +2 | tr -d '\r' > $dir/${language}.all_ses
grep -v -f $dir/spk_test $dir/${language}.all_ses | tr -d '\r'> $dir/spk_train
fi

for data_set in train test dev; do
echo $data_set
################
#echo "Split the speaker to train, test and dev"

#grep $language $dir/${language}.${data_set}_ses | cut -f2 | sed "s# #\n#g" | tail -n +2 | sort -u  > $dir/spk_${data_set}

#################
echo "rozdeleni genderu na test a train a dev"
echo $dir/spk_${data_set}
  grep -f $dir/spk_${data_set} $dir/all.gender | sort -u | tr '[:upper:]' '[:lower:]'  > $dir/${data_set}.gender

################
cut -d$'\t' -f3  $dir/filtered_content | sed s/.${language}A//g | grep -f $dir/spk_${data_set} | awk '{print $1," " substr ($1, 3, 4)}' | sort -u > $dir/${data_set}.utt2spk
perl utils/utt2spk_to_spk2utt.pl $dir/${data_set}.utt2spk | sort -u > $dir/${data_set}.spk2utt

#cut -d$'\t' -f2,3 $dir/filtered_content | tr '\t' '/' | sed 's/\\/\//g' | sed 's/^\/FIXED...\///g' |grep -f $dir/spk_${data_set} | awk -v var=$database '{print "| sox -t raw -r 8000 -e a-law -c 1 "var"/"$1" -t wav - |"}'| sort -u > $dir/${data_set}_part.scp

cut -d$'\t' -f2,3 $dir/filtered_content | tr '\t' '/' | sed 's/\\/\//g' | sed 's/^\/FIXED...\///g' |grep -f $dir/spk_${data_set} \
| awk -v var=$database '{print "sox -t raw -r 8000 -e a-law -c 1 "var"/"$1" -t wav -r 8000 -e signed-integer -c 1 - |"}'| sort -u > $dir/${data_set}_part.scp

paste $dir/${data_set}.utt2spk $dir/${data_set}_part.scp | awk '{$2=""; print $0}' > $dir/${data_set}.scp

###############
echo "Create transcripts files"
#perl local/speechdat_create_trans.pl $dir/${data_set}.utt2spk  $database $language | sort -k1  > $dir/${data_set}.txt 

[ -d $dir_work/data/$language/${data_set} ] && rm -r $dir_work/data/$language/${data_set}
mkdir -p $dir_work/data/$language/${data_set}

#cp $dir/${data_set}.scp $dir_work/data/$language/${data_set}/wav.scp || exit 1;
#cp $dir/${data_set}.txt $dir_work/data/$language/${data_set}/text || exit 1;
#cp $dir/${data_set}.spk2utt $dir_work/data/$language/${data_set}/spk2utt || exit 1;
#cp $dir/${data_set}.utt2spk $dir_work/data/$language/${data_set}/utt2spk || exit 1;
#cp $dir/${data_set}.gender $dir_work/data/$language/${data_set}/spk2gender || exit 1;

done

data_set="train"

cp $dir/${data_set}.scp $dir_work/data/$language/${data_set}/wav.scp || exit 1;
#cp $dir/${data_set}.txt $dir_work/data/$language/${data_set}/text || exit 1;
cp $dir/${data_set}.spk2utt $dir_work/data/$language/${data_set}/spk2utt || exit 1;
cp $dir/${data_set}.utt2spk $dir_work/data/$language/${data_set}/utt2spk || exit 1;
cp $dir/${data_set}.gender $dir_work/data/$language/${data_set}/spk2gender || exit 1;


for data_set in test dev; do
cat $dir/${data_set}.scp | grep "Z[0-9].${language}A\|S[0-9].${language}A" > $dir_work/data/$language/${data_set}/wav.scp 
#cat $dir/${data_set}.txt | grep 'S[0-9]\|Z[0-9]' > $dir_work/data/$language/${data_set}/text 
cat $dir/${data_set}.utt2spk | grep 'S[0-9]\|Z[0-9]' > $dir_work/data/$language/${data_set}/utt2spk 
perl utils/utt2spk_to_spk2utt.pl $dir_work/data/$language/${data_set}/utt2spk > $dir_work/data/$language/${data_set}/spk2utt

cut -d' ' -f1 $dir_work/data/$language/${data_set}/spk2utt > $dir_work/data/$language/${data_set}/tmp_spk
grep -f $dir_work/data/$language/${data_set}/tmp_spk $dir/${data_set}.gender  > $dir_work/data/$language/${data_set}/spk2gender
[ -f $dir_work/data/$language/${data_set}/tmp_spk ] && rm $dir_work/data/$language/${data_set}/tmp_spk
done






echo "Data preparation finished"
