#!/bin/bash

set -o errexit

function error_exit () {
  echo -e "$@" >&2; exit 1;
}

if [ $# -le 0 ]; then
   echo "Missing arguments. see ../run.sh for example."
   exit 1;
fi

language=$1;shift
dir_work=$1;shift

dir=$dir_work
dir_ml=$dir_work/data/ml/local
dir_dict=$dir_ml/dict


. ./path.sh # Needed for KALDI_ROOT

mkdir -p $dir_ml
mkdir -p $dir_dict


# merge lexicon
for lang in $language ; do
cat $dir_work/data/$lang/local/dict/lexicon.txt >> $dir_dict/lexicon.txt
cat $dir_work/data/$lang/local/dict/lexiconp.txt >> $dir_dict/lexiconp.txt
done

# prepare dict
cat $dir_dict/lexicon.txt | awk '{print $1="",$0}' | sed "s/ /\n/g" | sed '/^$/d' | sort -u > $dir_dict/phones.txt
cat $dir_dict/phones.txt | grep "sil" > $dir_dict/silence_phones.txt
cat $dir_dict/phones.txt | grep  -v "sil" > $dir_dict/nonsilence_phones.txt
echo "sil" > $dir_dict/optional_silence.txt
( tr '\n' ' ' < $dir_dict/silence_phones.txt; echo;
  tr '\n' ' ' < $dir_dict/nonsilence_phones.txt; echo;
) > $dir_dict/extra_questions.txt


for data_set in train test dev; do
    [ -d $dir_work/data/ml/$data_set ] && rm -r $dir_work/data/ml/$data_set
    mkdir -p $dir_work/data/ml/$data_set

    for lang in $language ; do
    cat $dir_work/data/$lang/$data_set/spk2gender | awk -v var="$lang" '{print $1 var,$1="",$0}' >> $dir_work/data/ml/$data_set/spk2gender_unsort

    cat $dir_work/data/$lang/$data_set/utt2spk | awk -v var="$lang" '{print $1 var, $2 var}' | awk '{print $2$1,$2}'  >> $dir_work/data/ml/$data_set/utt2spk_unsort

    cat $dir_work/data/$lang/$data_set/text | awk -v var="$lang" '{print $1 var,$1="",$0}' | awk '{print substr($1,3,4)substr($1,9,9)$1,$1="",$0}' >> $dir_work/data/ml/$data_set/text_unsort

    cat $dir_work/data/$lang/$data_set/wav.scp | awk -v var="$lang" '{print $1 var,$1="",$0}' | awk '{print substr($1,3,4)substr($1,9,9)$1,$1="",$0}' >> $dir_work/data/ml/$data_set/wav.scp_unsort
    done
    sort  -u $dir_work/data/ml/$data_set/spk2gender_unsort > $dir_work/data/ml/$data_set/spk2gender
    sort  -u $dir_work/data/ml/$data_set/utt2spk_unsort > $dir_work/data/ml/$data_set/utt2spk
    sort  -u $dir_work/data/ml/$data_set/text_unsort > $dir_work/data/ml/$data_set/text
    sort  -u $dir_work/data/ml/$data_set/wav.scp_unsort > $dir_work/data/ml/$data_set/wav.scp


    perl utils/utt2spk_to_spk2utt.pl $dir_work/data/ml/$data_set/utt2spk > $dir_work/data/ml/$data_set/spk2utt
done

echo "Data preparation finished"
